;
; On-board flash programmer on MachZ BUR environment for AT49F516 chip.
;
; (c)2000 ZF Linux Devices, Inc.
;
; Target Chip:	Atmel AT49F516
; Size:		64K
; Chipselect:	ms_cs0
; Mode:		16-bit
; Chip page:	00000000h
; Window base:	E0000h
; Window size:	10000h (64K)
;
; This code executes as BUR "Load and execute" function. It will fetch parameter
; record and payload code following to the executable code in dongle.
;
; Compile using MASM 6.11 with following command line:
;
;	ml /nologo /Sc /Fl /Zm /Fm /Ta AT49F516.ASM
;
; The resulting .COM file can be directly used as payload for 
; BUR "Uplaode and Execute" function
;

.MODEL TINY
.CODE
.486p

PRGBASE	= 0E0000h

		include FLASHPGM.MAC

		org	0
START:			
		push	cs
		pop	es
		
		SAYCRLF	'PGM - MachZ AT49F516 Flash Programmer V1.01'
		SAYCRLF	'----------------------------------------------'
		
		call	ZTPrepareRead
		
		; Go on with basket now	
		
		PROCESS_BASKET_HEADER	cs:[BasketSize], ExitPgmrFail
				
		; Create Memory window 
		
		MEMWIN	CS0, _16BIT_RW, PRGBASE, (64*1024), 0
		
		; Slow down ISA clock
		
		SLOW_ISA DIV_8		
		
		; Now we have chip layed out from E000:0 to E000:FFFF
		; Time to program some flash.
		
		; Check, whenever or not we have AT49F516 part on-board
		
		mov	al, 90h			; get ID
		call	Atmel_Cmd_16
		
		push	(PRGBASE / 10h)
		pop	ds		
		xor 	si, si
		mov	eax, ds:[si]		; get chip ID to EAX
		
		mov	ebx, eax		; save ID
						
		SAY	'Device: Mfg='
		call	SerOut8
		SAY	' DevID='
		shr	eax, 16
		call	SerOut8
		call	CRLF
		
		mov	al, 0F0h
		call	Atmel_Cmd_16		; end identification mode		
		
		and	ebx, 0FFFCFFFFh		; strip ID two low bits, since this may be the minor version code
		cmp	ebx, 00084001Fh		; check, if it's AT49F516
		jz	@f

		SAYCRLF	'This is not Atmel AT49F516'
		jmp	ExitPgmrFail				
@@:		
		; Handle the Basket data now
		
		call	ResetCRC
		
		PROCESS_RLE_HEADER ExitPgmrFail
		
		; We have right chip. Erase device now		
		
		SAY	'Erasing .. '
		
		mov	al, 80h			; chip erase command part 1
		call	Atmel_Cmd_16
		mov	al, 10h			; chip erase command part 2
		call	Atmel_Cmd_16
		
		; Erase is now in progress. Check, when we are ready.
		
		push	(PRGBASE/10h)
		pop	fs
		xor	si, si
		call	DSBX2Var
		mov	ax, 182	
		mov	ds:[bx.CountDown], ax	; gives 10sec. timeout	
			
@@:
		cmp	word ptr ds:[bx.CountDown], 0
		jz	@f		
		cmp	dword ptr fs:[si], 0FFFFFFFFh
		jnz	@b			; loop until erase completes
@@:
		mov	word ptr ds:[bx.CountDown], 0	; reset timer, so we will not loose our blinking LED's
		cmp	dword ptr fs:[si], 0FFFFFFFFh
		jz	@f
		
		SAYCRLF	'FAILED!'
		jmp	ExitPgmrFail
@@:		
		SAYCRLF	'OK!'
		
		; All set. Programming ...
		
		SAY	'Programming .. '
		
		mov	ecx, cs:[BasketSize]
		cmp	ecx, 0
		jz	PgmDone			; don't do anything if basket size is 0
		
		shr	ecx, 1			; divide by two to get word count
		
		push	(PRGBASE/10h)
		pop	ds
		xor	si, si
		
ProgramLoop:
		mov	al, 0A0h		; "program word" command
		call	Atmel_Cmd_16
		call	ZTRead_RLE		
		shl	ax, 8
		call	ZTRead_RLE
		xchg	al, ah			; byte order for 16-bit writing to memory
		mov	ds:[si], ax
		mov	bx, ax
		
		; on the extreme case we can have PCI backside clock 80Mhz. ISA
		; divider is 8, so we have 10M ISA bus clock. Programming cycle can be max.
		; 50us, so we need to wait here about 500 ISA cycles to kill that time.
		
		mov	di, 500
WaitWriting:
		cmp	ds:[si], bx
		jz	ByteOk
		dec	di		
		in	al, 80h			; create one ISA cycle
		jnz	WaitWriting
		
		SAYCRLF	'FAILED!'
		jmp	ExitPgmrFail		
ByteOk:		
		add	si, 2
		loop	ProgramLoop		
PgmDone:			
		SAYCRLF	'OK!'
		
		; Get original checksum
		
		call	ZTRead
		shl	ax, 8
		call	ZTRead
		xchg	al, ah
		
		call	DSBX2Var		; get variables block to DS:BX
		cmp	word ptr ds:[bx.YModemCRChi_C], ax
		jz	CRCOK
	
		SAY	'Data CRC failure: '		
		call	SerOut16
		mov	al, ' '
		call	SerSend2
		mov	ax, word ptr ds:[bx.YModemCRChi_C]
		call	SerOut16
		call	CRLF		
		jmp	ExitPgmrFail		
CRCOK:
		SAYCRLF	'Data CRC was OK!'
		
		jmp	ExitPgmrOk		
ExitPgmrFail:
		; Lit up RED LED and do not do anything else!
		
		mov	dx, 218h
		mov	al, 05Eh ;ZT_SIG_OUT
		out	dx, al
		inc	dx
		in	al, dx
		and	al, 11110101b
		or	al, ZT_LED_RED		;00001000b
		out	dx, al	
		
		SAYCRLF	'Freezing because of Programming failure ...'
		
		jmp	$			
ExitPgmrOk:
		; Lit up GREEN LED and continue with BUR		
		mov	dx, 218h
		mov	al, 05Eh ;ZT_SIG_OUT
		out	dx, al
		inc	dx
		in	al, dx
		and	al, 11110101b
		or	al, ZT_LED_GREEN	;00000010b
		out	dx, al
ExitPgmr:
		call	CRLF
		
		; Set timer to maximum value. This is useful to prevent
		; BUR from blinking with LED's when loading next commands.
		; This way we maintain our RED or GREEN LED setting
		
		call	DSBX2Var
		mov	word ptr ds:[bx.CountDown], 0FFFFh
		
		; Always exit with ZFL registers prepared for accelerated read!		
		call	ZTPrepareRead
				
		retf				; resume with BUR

		include BURAPI.ASM
		include FLASHPGM.ASM

END START
ENDS